/*
Copyright 2024 The Ceph-CSI Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package nfs

import (
	"testing"

	"github.com/stretchr/testify/require"
)

func TestNewCSIProvisionerRBAC(t *testing.T) {
	rbac, err := NewCSIProvisionerRBAC(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotNil(t, rbac)
	require.Equal(t, rbac.GetServiceAccount().Namespace, CSIProvisionerRBACDefaults.Namespace)
	require.Equal(t, rbac.GetServiceAccount().Name, CSIProvisionerRBACDefaults.ServiceAccount)
}

func TestNewCSIProvisionerRBACYAML(t *testing.T) {
	yaml, err := NewCSIProvisionerRBACYAML(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotEqual(t, "", yaml)
}

func TestNewClusterRole(t *testing.T) {
	cr, err := newClusterRole(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotNil(t, cr)
}

func TestNewClusterRoleBinding(t *testing.T) {
	crb, err := newClusterRoleBinding(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotNil(t, crb)
}

func TestNewRole(t *testing.T) {
	r, err := newRole(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotNil(t, r)
}

func TestNewRoleBinding(t *testing.T) {
	rb, err := newRoleBinding(CSIProvisionerRBACDefaults)

	require.NoError(t, err)
	require.NotNil(t, rb)
}
